<?xml version="1.0"?>

<!-- XUpdate to Sedna XQuery Update language -->

<xsl:stylesheet version="1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xupdate="http://www.xmldb.org/xupdate">

<xsl:output method="xml" version="1.0" omit-xml-declaration="yes" indent="no"/>

<xsl:strip-space elements="*"/>

<xsl:variable name="xupdate-namespace">http://www.xmldb.org/xupdate</xsl:variable>

<xsl:variable name="xp-prefix">
	<xsl:value-of select="xupdate:modifications/@xpathprefix" />
</xsl:variable>

<xsl:template match="xupdate:modifications">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="xupdate:insert-after">
UPDATE
insert <xsl:apply-templates />
following <xsl:value-of select="$xp-prefix" /><xsl:value-of select="@select" />
</xsl:template>

<xsl:template match="xupdate:insert-before">
UPDATE
insert <xsl:apply-templates />
preceding <xsl:value-of select="$xp-prefix" /><xsl:value-of select="@select" />
</xsl:template>

<xsl:template match="xupdate:rename">
UPDATE
rename <xsl:value-of select="$xp-prefix" /><xsl:value-of select="@select" /> on <xsl:apply-templates />
</xsl:template>

<xsl:template match="xupdate:remove">
UPDATE
delete <xsl:value-of select="$xp-prefix" /><xsl:value-of select="@select" />
</xsl:template>

<xsl:template match="xupdate:append/xupdate:element">
UPDATE
insert <xsl:element name="{@name}">
	<xsl:for-each select="./xupdate:attribute">
		<xsl:attribute name="{@name}">
			<xsl:apply-templates />
		</xsl:attribute>
	</xsl:for-each>
	<xsl:apply-templates />
</xsl:element>

<xsl:choose>
	<xsl:when test="@child">
		following <xsl:value-of select="$xp-prefix" /><xsl:value-of select="parent::xupdate:append/@select" />/*[<xsl:value-of select="parent::xupdate:append/@child" />]
	</xsl:when>
	<xsl:otherwise>
		following <xsl:value-of select="$xp-prefix" /><xsl:value-of select="parent::xupdate:append/@select" />/*[last()]
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template match="xupdate:append/xupdate:text">
UPDATE
insert text <xsl:value-of select="@name" /> { "<xsl:apply-templates />" }
<xsl:choose>
	<xsl:when test="parent::xupdate:append/@child">
		following
		<xsl:value-of select="$xp-prefix" /><xsl:value-of select="parent::xupdate:append/@select" />/*[<xsl:value-of select="parent::xupdate:append/@child" />]
	</xsl:when>
	<xsl:otherwise>
		following
		<xsl:value-of select="$xp-prefix" /><xsl:value-of select="parent::xupdate:append/@select" />/*[last()]
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template match="xupdate:variable">
declare variable $<xsl:value-of select="@name" /> := <xsl:value-of select="$xp-prefix" /><xsl:value-of select="@select" />;
</xsl:template>

<xsl:template match="xupdate:value-of">{<xsl:value-of select="@select" />}</xsl:template>

<xsl:template match="xupdate:append">
	<xsl:apply-templates />
</xsl:template>

<xsl:template match="xupdate:append/xupdate:attribute">
UPDATE
insert attribute <xsl:value-of select="@name" /> { "<xsl:apply-templates />" }
following <xsl:value-of select="$xp-prefix" /><xsl:value-of select="parent::xupdate:append/@select" />
</xsl:template>


<xsl:template match="xupdate:element">
<xsl:element name="{@name}">
	<xsl:for-each select="./xupdate:attribute">
		<xsl:attribute name="{@name}">
			<xsl:apply-templates />
		</xsl:attribute>
	</xsl:for-each>
	<xsl:apply-templates />
</xsl:element>
</xsl:template>

<xsl:template match="xupdate:text"><xsl:apply-templates /></xsl:template>

<xsl:template match="xupdate:processing-instruction">
<xsl:processing-instruction name="{@name}"><xsl:apply-templates /></xsl:processing-instruction>
</xsl:template>

<xsl:template match="xupdate:comment">
<xsl:comment><xsl:apply-templates /></xsl:comment>
</xsl:template>

<xsl:template match="*[namespace-uri(.)!=$xupdate-namespace]">
<xsl:element name="{name(.)}">
	<xsl:for-each select="@*">
		<xsl:attribute name="{name(.)}">
			<xsl:value-of select="." />
		</xsl:attribute>
	</xsl:for-each>
	<xsl:apply-templates />
</xsl:element>
</xsl:template>

<xsl:template match="*" />

<xsl:template match="xupdate:update">
<xsl:message terminate="yes">Updating nodes and attributes atomic values directly is not supported by Sedna.</xsl:message>
</xsl:template>

<xsl:template match="xupdate:cdata">
<xsl:message terminate="yes">Updating with CDATA is not supported by Sedna.</xsl:message>
</xsl:template>

<xsl:template match="xupdate:if">
<xsl:message terminate="yes">XUpdate "if" element is not defined well enough by XUpdate Working Group to implement.</xsl:message>
</xsl:template>

</xsl:stylesheet>