package DB_XML;

import org.basex.core.*;
import org.basex.core.proc.*;
import org.basex.io.*;

/**
 * This class demonstrates how new are created and deleted.
 *
 * @author Workgroup DBIS, University of Konstanz 2005-09, ISC License
 * @author BaseX Team
 */
public final class DBExample {
  /** Private constructor. */
  private DBExample() { }

  /**
   * Main method of the example class.
   * @param args (ignored) command-line arguments
   * @throws Exception exception
   */
  public static void main(final String[] args) throws Exception {
    // Creates a new database context, referencing the database.
    Context context = new Context();
    // Creates a standard output stream
    PrintOutput out = new PrintOutput(System.out);

    out.println("\n=== Create a database from a file:");

    // Sets an option: activates command info output
    new Set("info", false).execute(context, out);
    // Sets an option: chops whitespaces between text nodes (saves disk space)
    new Set("chop", true).execute(context, out);
    // Creates a database from the specified file.
    
    long start = System.currentTimeMillis();
    new CreateDB("./warehouse/f_sale.xml", "Exemple").execute(context, out);
    new CreateDB("./warehouse/d_supplier.xml", "Exemple1").execute(context, out);
    new CreateDB("./warehouse/d_customer.xml", "Exemple2").execute(context, out);
    new CreateDB("./warehouse/d_date.xml", "Exemple3").execute(context, out);
    new CreateDB("./warehouse/d_part.xml", "Exemple4").execute(context, out);
    long qtime = System.currentTimeMillis() - start;
	System.out.println(qtime);
    
    // Closes the database.
    new Close().execute(context, out);

        
    out.println("=== Drop databases:");

    // Removes the first database
    new DropDB("Exemple").execute(context, out);
    new DropDB("Exemple1").execute(context, out);
    new DropDB("Exemple2").execute(context, out);
    new DropDB("Exemple3").execute(context, out);
    new DropDB("Exemple4").execute(context, out);
    
    // Closes the output stream
    out.close();
    // Closes the database context
    context.close();
  }
}

