package DB_XML;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import Interface_XWeb.Xwebapp;

/**
 * Class abstraite grant la connection avec une base de donnes
 * 
 */
public abstract class DataBase
{
	protected String host;
	protected String user;
	protected String pass;
	protected String dbName;
	protected String fileName;

	/**
	 * Constructeur de la class abstract DataBase
	 */
	public DataBase()
	{
		try
		{
			this.host = Xwebapp.dbAddress.getText();
			System.out.println(this.host);
			this.dbName = Xwebapp.dbname.getText();
			System.out.println(this.dbName);
			this.user = Xwebapp.username.getText();
			System.out.println(this.user);
			this.pass = Xwebapp.pwd.getText();
			System.out.println(this.pass);
		} catch (Exception e)
		{
			Xwebapp.connection.setText("Champ vide!");
		}
	}

	/**
	 * Mutateur du adresse host
	 * 
	 * @param _host
	 *            adresse host
	 */
	public void setHost(String _host)
	{
		host = _host;
	}

	/**
	 * Mutateur de nom utilisateur
	 * 
	 * @param _user
	 *            nom utilisateur
	 */
	public void setUser(String _user)
	{
		user = _user;
	}

	/**
	 * Mutateur du mot de pass
	 * 
	 * @param _pass
	 *            mot de pass utlisateurs
	 */
	public void setPass(String _pass)
	{
		pass = _pass;
	}

	/**
	 * Mutateur du nom de la base
	 * 
	 * @param _dbName
	 *            nom de la base donnee
	 */
	public void setDbName(String _dbName)
	{
		dbName = _dbName;
	}

	/**
	 * Retourne une list qui contient les noms des differents bases
	 * 
	 * @return list qui contient les noms des differents bases
	 */
	public static ArrayList<String> getDbList()
	{
		ArrayList<String> l = new ArrayList<String>();
		l.add("exist");
		l.add("sedna");
		l.add("xindice");
		l.add("xhive");
		return l;
	}

	/**
	 * Fonction qui execute des requetes qui sont enregistres dans une List et
	 * calcule le temps d'execution de ces requetes
	 * 
	 * @param path
	 * @return list qui contient les message resultats
	 */
	public List<String> execCharge(List charge)
	{
		List<String> l = new ArrayList<String>();
		try
		{
			String str_tps = "0";
			String str_txt = "";
			l.add(str_tps);
			l.add(str_txt);

			// Rcuprer tous les lments contenant des requtes
			List namedChildren = charge;

			Iterator i = namedChildren.iterator();
			int cpt = 0;
			int cpt_echec = 0;
			long tps_total = 0;
			// Pour chaque requte
			while (i.hasNext())
			{
				Element courant = (Element) i.next();
				// On execute et on somme le temps d'xcution
				long t = executeReq(courant.getText());
				if (t != -1)
					tps_total += t;
				else
					++cpt_echec;
				System.out.println("Req " + cpt + " : " + t + " ms");
				++cpt;
			}
			System.out.println("Temps total : " + tps_total);
			if (tps_total != 0)
				str_tps = String.valueOf(tps_total);
			else
			str_tps = "-1";
			// Formatage des rsultats
			str_txt += "Temps d'xcution total : " + str_tps + " ms \n";
			str_txt += cpt + " requtes xcutes \n";
			str_txt += cpt_echec + " requtes choues\n";
			l.set(0, str_tps);
			l.set(1, str_txt);
			return l;
		} catch (Exception e){
			System.out.println("Une erreur est survenue lors de la lecture ou l'excution du fichier de charge");
			System.out.println(e.getMessage());
			l.set(0, "-1");
			return l;
		}
	}

	/**
	 * Fonction qui cree une connection vers une base
	 * 
	 * @return true si on est connecte
	 * @return false sinon
	 * @throws Exception
	 */
	public abstract boolean connection();

	/**
	 * Detruire la connection avec une base
	 * 
	 * @return true si on est deconnecte
	 * @return false sinon
	 */
	public abstract boolean deconnection();

	/**
	 * Ajouter un fichier dans la base de donnee
	 * 
	 * @param path
	 *            Chemin du fichier qui sera ajoute
	 * @param filename
	 *            Nom de la ressource
	 * @return true si l'insertion s'est droul correctement
	 * @return false sinon
	 */
	public abstract long addFile(String path, String filename);

	/**
	 * Executer une requete et retourne le temps d'execution
	 * 
	 * @param Req
	 *            Requte  excuter
	 * @return Le temps d'xcution de la requte (-1 si une erreur s'est
	 *         produite)
	 */
	public abstract long executeReq(String Req);

}
