package DB_XML;

import java.io.File;
import java.io.FileInputStream;

import javax.xml.transform.OutputKeys;

import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;
import org.xmldb.api.*;

// For the Xindice specific CollectionManager service
import org.apache.xindice.client.xmldb.services.*;

import org.apache.xindice.xml.dom.*;

import com.xhive.XhiveDriverFactory;
import com.xhive.core.interfaces.XhiveDriverIf;
import com.xhive.core.interfaces.XhiveSessionIf;

/**
 * Classe herite de la Classe Abstrait DataBase,permet de gerer la connection
 * avec une base de donnes Xindice
 * 
 */
public class Xindice extends DataBase
{

	private String driver = "org.apache.xindice.client.xmldb.DatabaseImpl";
	private Database db;

	public Xindice()
	{
		super();
	}

	public void creerCollection(String collection) throws Exception
	{
		Collection col = null;
		try
		{
			driver = "org.apache.xindice.client.xmldb.DatabaseImpl";
			Class c = Class.forName(driver);

			Database database = (Database) c.newInstance();
			DatabaseManager.registerDatabase(database);
			String uri = "xmldb:xindice:///db/";
			col = DatabaseManager.getCollection(uri);

			String collectionName = collection;
			CollectionManager service = (CollectionManager) col.getService(
					"CollectionManager", "1.0");

			// Build up the Collection XML configuration.
			String collectionConfig = "<collection compressed=\"true\" "
					+ "            name=\""
					+ collectionName
					+ "\">"
					+ "   <filer class=\"org.apache.xindice.core.filer.BTreeFiler\"/>"
					+ "</collection>";

			service.createCollection(collectionName, DOMParser
					.toDocument(collectionConfig));

			System.out.println("Collection " + collectionName + " created.");
		} catch (XMLDBException e)
		{
			System.out.println(e.getMessage());
		} finally
		{
			if (col != null)
			{
				col.close();
			}
		}
	}

	public boolean connection()
	{
		driver = "org.apache.xindice.client.xmldb.DatabaseImpl";
		Class c;
		boolean res = false;
		try
		{
			c = Class.forName(driver);
			db = (Database) c.newInstance();
			DatabaseManager.registerDatabase(db);
			String uri = host + "/" + dbName;
			Collection col = DatabaseManager.getCollection(uri);
			System.out.println(col);
			if (col == null)
			{
				res = false;
			} else
			{
				res = true;
			}
		} catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			res=false;
		}
		return res;
	}

	public boolean deconnection()
	{
		boolean res = false;
		try
		{
			DatabaseManager.deregisterDatabase(db);
			res = true;
		} catch (Exception e)
		{
			System.out.println(e.getMessage());
		}
		return res;
	}

	public long addFile(String path, String filename)
	{
		Collection col = null;
		long t1;
		long t2;
		try
		{
			t1 = System.currentTimeMillis();
			Class c = Class.forName(driver);
			Database database = (Database) c.newInstance();
			DatabaseManager.registerDatabase(database);
			col = DatabaseManager
					.getCollection(host + "/" + dbName, user, pass);

			XMLResource document = (XMLResource) col.createResource(filename,
					"XMLResource");

			// Lecture du fichier
			String f = readFileFromDisk(path);
			document.setContent(f);
			// Insertion du fichier dans la base
			col.storeResource(document);
			System.out.println("Load succeeded");
			System.out.println("=====================================\n");
			t2 = System.currentTimeMillis();
			return t2 - t1;
		} catch (Exception e)
		{
			System.out.println(e.getMessage());
			return -1;
		}
	}

	// Fonction qui lit un fichier et retourne le contenu
	private static String readFileFromDisk(String fileName) throws Exception
	{
		File file = new File(fileName);
		FileInputStream insr = new FileInputStream(file);
		byte[] fileBuffer = new byte[(int) file.length()];
		insr.read(fileBuffer);
		insr.close();
		return new String(fileBuffer);
	}

	public long executeReq(String Req)
	{

		Collection col = null;
		try
		{
			long t1;
			long t2;
			// initialise le driver
			Class c = Class.forName(driver);
			Database database = (Database) c.newInstance();
			DatabaseManager.registerDatabase(database);
			// Connection  la collection
			col = DatabaseManager
					.getCollection(host + "/" + dbName, user, pass);
			if (col == null)
				return -1;

			col.setProperty(OutputKeys.INDENT, "no");
			XPathQueryService service = (XPathQueryService) col.getService("XPathQueryService", "1.0");
			service.setProperty("indent", "yes");
			// Execution de la requete
			t1 = System.currentTimeMillis();
			ResourceSet resultSet = service.query(Req);
			
			
			 ResourceIterator results = resultSet.getIterator();
			 
			 while (results.hasMoreResources()) { 
				 Resource res =	 results.nextResource(); 
				 //System.out.println((String)res.getContent()); 
			 }
			 
			 t2 = System.currentTimeMillis();
			
			return t2 - t1;
		} catch (Exception e)
		{
			System.out.println(e.getMessage());
			return -1;
		} finally
		{
			if (col != null)
			{
				try
				{
					col.close();
				} catch (XMLDBException e)
				{
					System.out.println(e.getMessage());
				}
			}
		}
	}

	/*
	 * public static void main(String arg[]) { Xindice x = new Xindice(); try {
	 * x.creerCollection("bianc"); // System.out.print(x.deconnection()); //
	 * System.out.print(x.connection()); System.out.print(x
	 * .addFile("E:/Workspace/XWeb_Final/charge/charge1.xml", "charge2.xml")); }
	 * catch (Exception e) { e.printStackTrace(); } }
	 */

}
