package Outils;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import org.jaxen.*;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * Classe hrite de la classe PrintWriter, permet de ajouter les absences,les
 * permutations, les granulations sur les dimensions generees en utilisant les
 * probabilites.
 * 
 */
public class PrintWriterXWeb extends PrintWriter
{
	// ######################################################################
	// Dclaration des instances
	// ######################################################################

	double p_permut = 0;// paramtre de Bernouli pour la permutation

	double p_absence = 0;// paramtre de Bernouli pour l'absence

	Vector v_permut;// Ensemble de combinaison de permutation

	Vector v_absence;// Ensemble de combinaison de absence

	Vector v_donnees;// Les lments

	int i;// nombre d'lment

	double p_granul = 0;

	HashMap paths;

	public static String repertoire = "";

	// ######################################################################
	// Le Constructeur
	// ######################################################################

	/**
	 * Constructeur de la class PrintWriterXWeb
	 * 
	 * @param fw
	 * @param i
	 *            nombre de chaine carateres qui sera ecrits en fonction d'un
	 *            parametre de presence et d'un parametre de permutation
	 */
	public PrintWriterXWeb(FileWriter fw, int i)
	{

		super(fw);
		this.i = i;
		v_absence = tableau_binaire(i);
		v_permut = permuterIndice(i);
		// initialiser le vecteur v_donnees, si on ne remplir pas v_donnees
		// avec
		// une valeur,
		// il aura une exception plus tarde quand on applique methode set
		v_donnees = new Vector();
		v_donnees.setSize(i);
		paths = dimElements();
	}

	/**
	 * Fonction parseur d'un fichier XML
	 * 
	 * @param path
	 *            chemin du fichier XML a parser
	 * @return racine du fichier XML parse
	 */
	public static Element parse(String path)
	{
		org.jdom.Document document = new Document();
		Element racine;

		SAXBuilder sxb = new SAXBuilder();
		try
		{

			document = sxb.build(new File(path)); // "dw-model.xml"
			racine = document.getRootElement();
			return racine;
		} catch (Exception e)
		{
			System.out.println(e.getMessage());
			return null;
		}

	}

	// ######################################################################
	// Les mutateurs
	// ######################################################################

	/**
	 * Mutateur de parametre permutation(p_permut)
	 * 
	 * @param p
	 *            la probabilite d'avoir les permutations sur les dimensions ou
	 *            les mesures generees
	 */
	public void set_p_permut(double p)
	{
		this.p_permut = p;
	}

	/**
	 * Mutateur de parametre absence(p_absence)
	 * 
	 * @param p
	 *            la probalite d'avoir les absences sur les dimensions ou les
	 *            mesures generees
	 */
	public void set_p_absence(double p)
	{
		this.p_absence = p;
	}

	/**
	 * Mutateur de parametre granularite(p_granularite)
	 * 
	 * @param p
	 *            la probabilite d'avoir une dimension moins fine
	 */
	public void set_p_granul(double p)
	{
		this.p_granul = p;
	}

	/**
	 * Mthode qui permet d'ajouter un lment dans objet PrintWriterXWeb sans
	 * tenir compte de la granularite
	 */
	public void set(int indice, String chaine)
	{
		v_donnees.setElementAt(chaine, indice);
	}

	/**
	 * Mthode qui permet d'ajouter un lment dans objet PrintWriterXWeb en
	 * appliquant un granularite si granularite = true
	 */
	public void set(int indice, String level_name, String id)
	{
		if (bernoulli(p_granul))
		{
			String path = "";// nom du fichier de la dimension
			String s = granularite(level_name, id);

			v_donnees.setElementAt(s, indice);
		} else
		{
			String s1 = "<dimension dim-id='" + level_name + "' value='"
					+ id + "'/>";
			v_donnees.setElementAt(s1, indice);
		}

	}

	/**
	 * Mthode qui ecrit les elements dans le fichier XML en tenant compte des
	 * parmtres de probabilit p_permut et p_absence
	 */
	public void printXWeb()
	{
		Vector v = (Vector) this.v_donnees.clone();
		Vector v_p = (Vector) this.v_permut.clone();
		
			v = abscence();
			v_p = permuterIndice(v.size()); // v_p est un vector
			// intermediaire de v_permut

		if (bernoulli(this.p_permut))
		{
			if (v_p.size() != 0)
			{
				Random r = new Random();
				int unif = r.nextInt(v_p.size()); // generer un nombre qui
				// suit loi uniforme
				Vector temp = (Vector) v_p.get(unif);
				int j = 0;
				while (j < temp.size())
				{
					int k = (Integer) temp.get(j); // recuperer la valeur du
					// rang j
					println(v.get(k));
					j++;
				}
			} else
			{
				println("<!-- Absence total !!!!!! -->");
			}
		} else
		{
			Iterator it2 = v.iterator();
			while (it2.hasNext())
			{
				println(it2.next());
			}
		}
	}

	/**
	 * retourne une HashMap qui contient les level_id(key) et les chemins de
	 * leur fichiers dimensions(value)
	 * 
	 * @param level_id
	 * @return une HashMap qui contient 'keys'=>les nom des levels ,
	 *         'values'=>les fichiers dimensions auquels les levels appartient
	 */
	public HashMap dimFichierPath()
	{
		HashMap res = new HashMap();
		Element racine = parse("dw-model.xml");
		try
		{
			org.jdom.xpath.XPath xpa = org.jdom.xpath.XPath
					.newInstance("//Level");
			List levels = xpa.selectNodes(racine);
			Iterator it = levels.iterator();

			while (it.hasNext())
			{
				Element level_courant = (Element) it.next();
				String level_id = level_courant.getAttributeValue("id");
				String path = searchPath(searchDimension(level_id)); // trouver
				// le
				// chemin
				// du
				// fichier
				// Dimension
				// pour level courant
				res.put(level_id, filtrePath(path)); // filtrer '\' et
				// ajouter
				// dans HashMap
			}

		} catch (JDOMException e)
		{
			System.out.println("Erreur JDOM " + e.getMessage());
			System.out.println(e.getMessage());
		}
		return res;
	}

	/**
	 * retourne une HashMap qui contient les level_id(key) et les dimensions
	 * Element(s) correspondantes(value)
	 * 
	 * @return une HashMap qui contient 'keys'=>les nom des levels ,
	 *         'values'=>les elements racines des fichiers dimensions(xml)
	 *         auquels les levels appartients
	 */
	public HashMap dimElements()
	{
		HashMap res = new HashMap();
		Element racine = parse("dw-model.xml");
		try
		{
			org.jdom.xpath.XPath xpa = org.jdom.xpath.XPath
					.newInstance("//Level");
			List levels = xpa.selectNodes(racine);
			Iterator it = levels.iterator();

			while (it.hasNext())
			{
				Element level_courant = (Element) it.next();
				String level_id = level_courant.getAttributeValue("id");
				String path = searchPath(searchDimension(level_id));
				Element dim_racine = parse(path);
				res.put(level_id, dim_racine);
			}

		} catch (JDOMException e)
		{
			System.out.println("Erreur JDOM " + e.getMessage());
			System.out.println(e.getMessage());
		}
		return res;
	}

	// ######################################################################
	// Fonctions statistiques
	// ######################################################################

	// Fonction qui retourne une varible booleen qui suit un loi bernoulli
	// de parametre P
	// Origine: Universite Princetone
	private boolean bernoulli(double p)
	{
		return Math.random() < p;
	}

	// Return an integer uniformly between 0 and N-1
	private static int uniform(int N)
	{
		return (int) (Math.random() * N);
	}

	// ######################################################################
	// Absence--Permutation--Granularite
	// ######################################################################

	//
	private static Vector permuterIndice(int n)
	{
		Vector source;
		if (n <= 0)
		{
			source = new Vector();
			return source;
		} else
		{
			source = new Vector(n);
			int i = 0;
			while (i < n)
			{
				source.add(i);
				i++;
			}
			PermutationGenerator x = new PermutationGenerator(source.size());
			Vector V = new Vector();
			while (x.hasMore())
			{
				Vector v = new Vector();
				int[] temp = x.getNext();
				for (i = 0; i < temp.length; i++)
				{
					v.add(source.get(temp[i]));
				}
				V.add(v);
			}
			return supprimLeOrdonne(V);
		}
	}

	// Methode qui supprime les elements en fonction du code binaire contenu
	// dans le vecteur v en parametre
	private Vector abscences(Vector v1)
	{// v contient les donnee a ecrire v1
		// contient le code des abscences
		Random r = new Random();
		Vector v = (Vector) v_donnees.clone();

		int u = r.nextInt(v1.size()); // x: variable aleatoire
		// uniforme[1,{taille du vecteur}]
		String s = (String) v1.get(u); // transtipage en String car il n'y a
		// que des char '1' ou '0'
		int i = 0;
		while (i < s.length())
		{// bouche qui ecrit les valeurs abscente
			if (s.charAt(i) == '0')
			{// condition pour ecrire l'object ou pas
				v.removeElementAt(i);
				if (i == 0)
				{
					s = s.substring(1, s.length());
				} else if (i == s.length() - 1)
				{
					s = s.substring(0, i - 1);
				} else
				{
					s = s.substring(0, i - 1) + s.substring(i + 1, s.length());
				}
				i--;
			}
			i++;
		}
		return v;
	}

	//Absence appliqu sur chaque dimension individuelement
	private Vector abscence()
	{
		Vector v = (Vector) v_donnees.clone();
		Vector res = new Vector();
		Iterator it = v.iterator();
		while (it.hasNext())
		{
			String courant = (String)it.next();
			boolean b = bernoulli(this.p_absence);
			if (!b)
			{
				res.add(courant);
			}
		}
		return res;
	}

	// Ecrire une dimension avec un granularite hierarchique
	private String granularite(String level_name, String id)
	{

		// nombre de level hierarchie a partir du dimension_name donnee
		Element racine = (Element) paths.get(level_name);
		int nbLevel = nbLevelHierachie(level_name, id, racine);
		// Envoi de la hierarchie selectionner a partir d'une loi donne

		// Condition pour supprimer les non granularits
		int iAleatoire = 0;
		if (nbLevel > 0)
		{
			iAleatoire = uniform(nbLevel) + 1;
		}

		// Remonte de hierarchie en retournant un chaine de caractere;

		String chaine = remonteHr(level_name, id, iAleatoire, racine);
		return chaine;
	}

	// ######################################################################
	// Fonctions privees utilisees par permuterIndice(int)
	// ######################################################################

	// Fonction permet de tester si les contenu du vector sont ordonnes.
	private static boolean estOrdonnee(Vector v)
	{
		boolean res = true;
		if (v.size() > 1)
		{
			int i = 0;
			int j = 1;
			int o1 = (Integer) v.get(i);// ((Integer) v.get(i)).intValue();
			int o2 = (Integer) v.get(j);
			((Integer) v.get(j)).intValue();

			while (j < v.size())
			{
				o1 = (Integer) v.get(i);
				o2 = (Integer) v.get(j);
				if (o1 > o2)
				{
					res = false;
					i++;
					j++;
				} else
				{
					i++;
					j++;
				}
			}
		}
		return res;
	}

	// Fonction permet de supprimer l'occurence ordonne d'un vector.
	private static Vector supprimLeOrdonne(Vector V)
	{
		Vector res = new Vector();
		if (V.size() <= 1)
		{
			res = V;
		} else
		{
			Iterator it = V.iterator();
			Vector v;
			while (it.hasNext())
			{
				v = (Vector) it.next();
				if (!estOrdonnee(v))
				{
					res.add(v);
				}
			}
		}
		return res;
	}

	// ######################################################################
	// Fonctions privees utilisees par absence(Vector)
	// ######################################################################

	/*
	 * Renvois l'ensemble des combinaison binaire des absences en fonction du n
	 * d'element Par exemple: si n=2 la fonction nous renvois [00, 01, 10] si
	 * n=3 la fonction nous renvois [000, 001, 010, 011, 100, 101, 110] si n=4
	 * la fonction nous renvois [0000, 0001, 0010, 0011, 0100, 0101, 0110, 0111,
	 * 1000, 1001, 1010, 1011, 1100, 1101, 1110] Exmeple combinaison binaire:
	 * [1,0,1] Presence du premier(rang 0) et dernier element(rang 2), absences
	 * du deuxieme (rang 1)
	 */
	private static Vector tableau_binaire(int n)
	{
		int i = 0;
		int taille = (int) (Math.pow(2, n) - 1);// formule qui donne le nombre
		// de combinaison
		Vector vector = new Vector(); // vector qui contiendra les donnees de la
		// representation bynaire
		int maxBin = Integer.toBinaryString(taille).length();// maximum de
		// chiffre
		// binaire
		// byte[] tableau=new byte[maxBin];

		while (i < taille)
		{// boucle principal qui entre les valeurs i binaire (represent en
			// chaine de caractere) dans le vector

			String S = Integer.toBinaryString(i);// valeur courant
			for (int k = S.length(); k < maxBin; k++)
			{// boucle qui permet d'ajouter des '0' complementaire pour
				// completer
				S = '0' + S;
			}

			vector.add(S);// ajout du tableau binaire dans le vecteur
			i++;
		}
		return vector;
	}

	// ######################################################################
	// Fonctions privees utilisees par granularite(String,String)
	// ######################################################################

	/*
	 * level_: le nom de ma dimension hierarchique id____: l'identifiant de
	 * l'occurance i_____: le nombre de hierarchie  remonter path__: chemin du
	 * fichier xml ou se trouve les occurences de la dimension
	 */
	private String remonteHr(String level, String id, int i, Element racine)
	{
		if (i == 0)// Cas d'arret
		{
			return "<dimension dim-id='" + level + "' value='" + id + "'/>";
		}

		else
		{// recursivit
			List dimensions = racine.getChildren();
			Iterator it = dimensions.iterator();

			while (it.hasNext())
			{// parcours des levels
				Element courant = (Element) it.next();
				String id_level = courant.getAttributeValue("id");
				if (id_level.equals(level))
				{

					List occurrences = courant.getChildren();
					Iterator it1 = occurrences.iterator();

					while (it1.hasNext())
					{// parcours des occurences(element) du level courant
						Element courant1 = (Element) it1.next();
						String id_occ = courant1.getAttributeValue("id");

						if (id_occ.equals(id))
						{// *********instance trouver********

							String uplevel = courant1
									.getAttributeValue("rollup");// dans
							// l'attribut
							// de
							// l'occurence
							String uplevel_id = uplevel_id(level, uplevel); // dans
							// la balise
							// <attribut/>
							// dans
							// l'occurance

							List attributs_instance = courant1.getChildren();
							Iterator it2 = attributs_instance.iterator();

							while (it2.hasNext())
							{// parcours des elements de l'instance
								// <attribute>
								Element courant2 = (Element) it2.next();

								if (courant2.getAttributeValue("id").equals(
										uplevel_id))
								{// cl exterieur identifier

									String value = courant2
											.getAttributeValue("value");

									i--;
									return remonteHr(uplevel, value, i, racine); // application
									// sur
									// le
									// level
									// au
									// dessus(uplevel)
								}

							}

						}

					}
				}

			}
			return "aucune hierarchie";
		}
	}

	// Fonction qui permet de renvoyer une chaine qui correspond a l'identifiant
	// de la cl exterieur vers les hierarchies superieur
	private String uplevel_id(String level, String uplevel)
	{
		char c = level.charAt(0);
		return (c + "_" + uplevel + "key").toLowerCase();
	}

	// Fonction qui calcule le nombre level hierachique superieur pour une
	// instance donnee
	private int nbLevelHierachie(String level_id, String instance_id,
			Element racine_dimension)
	{
		String rollup = "";
		String fk_value = "";
		// System.out.println("level_id: "+level_id);
		// System.out.println("isntance_id: "+instance_id);
		// System.out.println("dimension_path: "+dimension_path);

		List levels = racine_dimension.getChildren("Level");
		Iterator it = levels.iterator();
		while (it.hasNext())
		{ // parcourir les levels
			Element level_courant = (Element) it.next();

			if (level_courant.getAttributeValue("id").equals(level_id))
			{

				List instances = level_courant.getChildren("instance");
				Iterator it1 = instances.iterator();
				while (it1.hasNext())
				{ // parcourir les instances
					Element instance_courant = (Element) it1.next();
					if (instance_courant.getAttributeValue("id").equals(
							instance_id))
					{

						rollup = instance_courant.getAttributeValue("rollup");

						List attributs = instance_courant
								.getChildren("attribute");
						Iterator it2 = attributs.iterator();
						while (it2.hasNext())
						{ // parcourir les attributs
							Element attribut_courant = (Element) it2.next();
							String fk = uplevel_id(level_courant
									.getAttributeValue("id"), rollup);

							if (attribut_courant.getAttributeValue("id")
									.equals(fk))
							{

								fk_value = attribut_courant
										.getAttributeValue("value");
							}
						}
					}
				}
			}
		}
		if (rollup.equals("")) // cas d'arret de la recursivite
		{
			return 0;
		} else
		{
			return 1 + nbLevelHierachie(rollup, fk_value, racine_dimension); // recursivite
		}
	}

	// ######################################################################
	// Fonctions privees utilisees par dimFichierPath() et dimElements()
	// ######################################################################

	// retourne le chemin d'une fichier dimension qui correspond la dimension
	// donnee
	private static String searchPath(String dim_nom)
	{
		String res = "";
		File[] files = null;

		File directoryToScan = new File(repertoire);
		files = directoryToScan.listFiles();// rcuprer un tableau qui
		// contient
		// tout les fichiers
		// du rpertoire donn.
		int i = 0;
		while (i < files.length)
		{
			String path = files[i].getPath(); // recuperer le chemin du
			// fichier
			// courant
			if (path.contains(dim_nom))
			{
				res = path;
				return res;
			} else
			{
				i++;
			}
		}
		return res;
	}

	// retourne la dimension a la quelle level donnee appartient.
	private static String searchDimension(String level_id)
	{
		String res = "";

		Element racine = parse("dw-model.xml");
		List dimensions = racine.getChildren();
		Iterator it = dimensions.iterator();
		while (it.hasNext())
		{
			Element dimension_courant = (Element) it.next();
			List levels = dimension_courant.getChildren("Level");
			Iterator it1 = levels.iterator();
			while (it1.hasNext())
			{
				Element level_courant = (Element) it1.next();
				if (level_courant.getAttributeValue("id").equals(level_id))
				{
					res = dimension_courant.getAttributeValue("id");
					return res;
				}
			}
		}
		return res;
	}

	// Filtrer '\' du chemin donnee et le remplace par '/'
	private static String filtrePath(String path)
	{
		String res = "";
		int i = 0;
		i = path.indexOf("\\");
		res = path.substring(0, i) + "/" + path.substring(i + 1, path.length());
		return res;
	}

	public static void main(String[] args)
	{
		/*
		 * PrintWriterXWeb pw = null; try { pw = new PrintWriterXWeb( new
		 * FileWriter("warehouse/FactDoc_new.xml"), 6); pw.set_p_permut(0.5);
		 * pw.set_p_absence(0.5); } catch (Exception e) { } int i = 0; while (i
		 * < 1000) { pw.set(0, "<dimension dim-id='Date' value-id='" + i +
		 * "'/>"); pw.set(1, "<dimension dim-id='Customer' value-id='" + i +
		 * "'/>"); pw.set(2, "<dimension dim-id='Part' value-id='" + i + "'/>");
		 * pw.set(3, "<dimension dim-id='Suppliers' value-id='" + i + "'/>");
		 * pw.set(4, "<dimension dim-id='Vendeur' value-id='" + i + "'/>");
		 * pw.set(5, "<dimension dim-id='Magasin' value-id='" + i + "'/>");
		 * pw.printXWeb();
		 * System.out.println("********************************************");
		 * i++; }
		 */
	}

}
